/* REXX                                                              */
/*                                                                   */
/* Licensed materials - Property of IBM                              */
/* 5724-V04  Copyright IBM Corp. 2011                               */
/* All rights reserved                                               */
/* US Government users restricted rights  -  Use, duplication or     */
/* disclosure restricted by GSA ADP schedule contract with IBM Corp. */
/*                                                                   */
/* Rational Team Concert IVP                                         */
/*===================================================================*/
parse arg args
/* get pathnames allocated in the JCL */
call bpxwdyn "info fi(conf) inrtpath(confpath)"
call bpxwdyn "info fi(work) inrtpath(workpath)"
path   = ''
log    = ''
file   = 0
max_rc = 0
Do While args <> ''
  parse var args xarg args
  Select
    When translate(xarg) = '-DEBUG'  then parm=parm '-DEBUG'
    When translate(xarg) = '-FILE'   then file=1
    Otherwise                             NOP
  End    /* select */
End    /* Do while */

If right(confpath,1) <> '/' then confpath=confpath'/'
If confpath = '' then
  confpath = '/etc/jazz/ccm/'

Say '********************************************************************'
Say '* Rational Team Concert - Installation Verification Process (IVP)  *'
Say '********************************************************************'
Say ' '
Say 'INFO : RTC Configuration directory set to : 'confpath
Say 'INFO : RTC Work directory set to          : 'workpath
Say ' '

Call General
Call ISPFGateway
Call BFAgent
Call ISPFClient

Say '******************************************************************'
Say '* Rational Team Concert - Installation Verification Complete     *'
Say '******************************************************************'

Exit max_rc

General :

Say ' '
Say '------------------------------------------------------------------'
Say '- Checking the general configuration                             -'
Say '------------------------------------------------------------------'
Say ' '

Say 'INFO : Checking if /tmp is writable'
Say ' '
shellcmd = 'ls -alEd /tmp'
sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
If sh_rc = 0 Then
Do
  Say Copies('-',80)
  Do T = 1 to stdout.0
    parse var stdout.T perms 13 ext 17 . 21 owner 29 group,
                       38 size 47 date 60 filename
    Say stdout.T
    If substr(perms,3,1) /= 'w' |,
       substr(perms,6,1) /= 'w' |,
       substr(perms,9,1) /= 'w' then
    Do
      Say ' '
      Say 'WARNING : /tmp may not be writable. If you are using    '
      Say '          a different directory instead of /tmp ensure  '
      Say '          the TMPDIR environment variable is set in all '
      Say '          required shell scripts.                       '
      Say ' '
    End
    Else
    Do
      Say ' '
      Say 'INFO : /tmp is writable.'
      Say ' '
    End
  End
  Say Copies('-',80)
  Say ' '
End

Return

ISPFGateway :

Say '------------------------------------------------------------------'
Say '- Checking the configuration of the ISPF Gateway                 -'
Say '------------------------------------------------------------------'
Say ' '

shellcmd='find' confpath'startispf.sh'
sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
If sh_rc /= 0 Then
Do
  Call Errors
  Say ' '
  Say 'ERROR:' confpath'startispf.sh cannot be accessed'
  Say 'ERROR:  Have you run job BLZCPBTK?'
  sh_rc = 8
  If sh_rc > max_rc then max_rc = sh_rc
End

/* read startispf.sh to get the PATH export */
filein = confpath'startispf.sh'
fileline.0 = 0
Address syscall "readfile (filein) fileline."
If rc /= 0 Then
Do
  Call Errors
End
Else
Do
  pathexp = ''
  Do i = 1 to fileline.0 while (pathexp = '')
    Parse var fileline.i firstbit 'export PATH=' pathexp
    /* ignore any comments  */
    if pos('#',firstbit) /= 0 Then pathexp = ''
  End
  If pathexp = '' Then
  Do
    Say 'INFO : PATH does not seem to be set in startispf.sh'
    pathexp = '$PATH'
  End
  shellcmd = 'export PATH='pathexp';echo $PATH'
  sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
  If sh_rc /= 0 Then
  Do
    Call Errors
    Say ' '
    Say 'ERROR: Error setting PATH export.'
    sh_rc = 8
    If sh_rc > max_rc then max_rc = sh_rc
  End
  Do T = 1 to stdout.0
     Say 'INFO : Path being set to 'stdout.T
     Say ' '
  End
  /* checking that ISPZINT can be invoked */
  shellcmd = 'export PATH='pathexp';whence -v ISPZINT'
  sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
  If sh_rc = 0 Then
  Do
    parse var stdout.1 'ISPZINT is ' pathint 'ISPZINT'
    Say 'INFO : ISPZINT invoked from 'pathint
    Say 'INFO : Checking permissions of ISPF gateway files'
    Say ' '
    shellcmd = 'cd 'pathint';ls -alE'
    sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
    If sh_rc = 0 Then
    Do
      Say Copies('-',80)
      Do T = 1 to stdout.0
         parse var stdout.T perms 13 ext 17 . 21 owner 29 group,
                            38 size 47 date 60 filename
         If substr(perms,1,1) /= 'd' &,
            substr(perms,1,5) /= 'total' then
           Say stdout.T
         /* check the sticky bit is on for ISPZINT and ISPZTSO */
         If (Strip(filename) = ISPZINT | Strip(filename) = ISPZTSO ) Then
           If Substr(perms,10,1) /= 't' Then
             Say 'ERROR: Sticky bit not set on file : 'filename

         /* check the sticky bit is off for the rest           */
         If (Strip(filename) /= ISPZINT & Strip(filename) /= ISPZTSO ) Then
           If Substr(perms,10,1) = 't' Then
             Say 'ERROR: Sticky bit incorrectly set on file : 'filename
      End
      Say Copies('-',80)
      Say ' '
    End
    Else
    Do
      Say 'ERROR: Problem checking permissions of files in 'pathint
      Say ' '
      Call Error
    End
  End
  Else
  Do
    Say 'ERROR: ISPZINT not found. Check ISPF Gateway is set in PATH statement'
    Say ' '
  End
  /* checking where ISPZXENV is invoked from*/
  shellcmd = 'export PATH='pathexp';whence -v ISPZXENV'
  sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
  If sh_rc = 0 Then
  Do
    parse var stdout.1 'ISPZXENV is ' pathenv 'ISPZXENV'
    Say 'INFO : ISPZXENV invoked from 'pathenv
    If pathenv /= pathint then
    Do
      Say 'INFO : Checking permissions of configured ISPF gateway files'
      Say ' '
      shellcmd = 'cd 'pathenv';ls -alE'
      sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
      If sh_rc = 0 Then
      Do
        Say Copies('-',80)
        Do T = 1 to stdout.0
           parse var stdout.T perms 13 ext 17 . 21 owner 29 group,
                              38 size 47 date 60 filename
           If substr(perms,1,1) /= 'd' &,
             substr(perms,1,5) /= 'total' then
             Say stdout.T
           /* check the sticky bit is on for ISPZINT and ISPZTSO */
           If (Strip(filename) = ISPZINT | Strip(filename) = ISPZTSO ) Then
             If Substr(perms,10,1) /= 't' Then
               Say 'ERROR: Sticky bit not set on file : 'filename

         /* check the sticky bit is off for the rest           */
           If (Strip(filename) = ISPZXENV | Strip(filename) /= ISPZXML ) Then
             If Substr(perms,10,1) = 't' Then
               Say 'ERROR: Sticky bit incorrectly set on file : 'filename
        End
        Say Copies('-',80)
        Say ' '
      End
    End

    /* get the values of the environment variables set in ISPZXENV */
    /* read ISPZXENV to get the environment variables */
    filein = pathenv'ISPZXENV'
    fileline.0 = 0
    Address syscall "readfile (filein) fileline."
    If rc /= 0 Then
      Call Errors
    Else
    Do
      steplib  = ''
      ispfwork = ''
      ispfconf = ''
      Do i = 1 to fileline.0
        If steplib = '' then
          Parse var fileline.i 'STEPLIB = 'steplib
        If ispfconf = '' then
          Parse var fileline.i "CGI_ISPCONF = '"ispfconf"'"
        If ispfwork = '' then
          Parse var fileline.i "CGI_ISPWORK = '"ispfwork"'"
      End
      Say 'INFO : Environment variable STEPLIB set to     : 'steplib
      Say 'INFO : Environment variable CGI_ISPCONF set to : 'ispfconf
      Say 'INFO : Environment variable CGI_ISPWORK set to : 'ispfwork
      Say ' '

      If right(ispfconf,1) <> '/' then ispfconf=ispfconf'/'

      shellcmd='find' ispfconf'ISPF.conf'
      sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
      If sh_rc /= 0 Then
      Do
        Call Errors
        Say 'ERROR:' ispfconf'ISPF.conf not found'
        Say 'ERROR: Have you run job configured the ISPF gateway ' ||,
                'the ISPF Planning and Customization Guide?'
        Say ' '
        sh_rc = 8
        If sh_rc > max_rc then max_rc = sh_rc
      End
      /* read ISPF.conf to check all data sets exist */
      filein = ispfconf'ISPF.conf'
      fileline.0 = 0
      Address syscall "readfile (filein) fileline."
      If rc /= 0 Then
        Call Errors
      Else
      Do
        Say 'INFO : Checking ISPF.conf allocations'
        sysproc = ''
        sysexec = ''
        ispplib = ''
        ispmlib = ''
        isptlib = ''
        ispslib = ''
        ispllib = ''
        dsn_rc  = 0
        Do i = 1 to fileline.0
          If sysproc = '' then
          Do
            Parse var fileline.i firstbit 'sysproc='sysproc
            if pos('*',firstbit) /= 0 Then sysproc = ''
            If sysproc /= '' Then
            Do
              Call Chk_dsns 'sysproc' sysproc
              If rc > dsn_rc then dsn_rc = rc
            End
          End
          If sysexec = '' then
          Do
            Parse var fileline.i firstbit 'sysexec='sysexec
            if pos('*',firstbit) /= 0 Then sysexec = ''
            If sysexec /= '' Then
            Do
              Call Chk_dsns 'sysexec' sysexec
              If rc > dsn_rc then dsn_rc = rc
            End
          End
          If ispplib = '' then
          Do
            Parse var fileline.i firstbit 'ispplib='ispplib
            if pos('*',firstbit) /= 0 Then ispplib = ''
            If ispplib /= '' Then
            Do
              Call Chk_dsns 'ispplib' ispplib
              If rc > dsn_rc then dsn_rc = rc
            End
          End
          If ispmlib = '' then
          Do
            Parse var fileline.i firstbit 'ispmlib='ispmlib
            if pos('*',firstbit) /= 0 Then ispmlib = ''
            If ispmlib /= '' Then
            Do
              Call Chk_dsns 'ispmlib' ispmlib
              If rc > dsn_rc then dsn_rc = rc
            End
          End
          If isptlib = '' then
          Do
            Parse var fileline.i firstbit 'isptlib='isptlib
            if pos('*',firstbit) /= 0 Then isptlib = ''
            If isptlib /= '' Then
            Do
              Call Chk_dsns 'isptlib' isptlib
              If rc > dsn_rc then dsn_rc = rc
            End
          End
          If ispslib = '' then
          Do
            Parse var fileline.i firstbit 'ispslib='ispslib
            if pos('*',firstbit) /= 0 Then ispslib = ''
            If ispslib /= '' Then
            Do
              Call Chk_dsns 'ispslib' ispslib
              If rc > dsn_rc then dsn_rc = rc
            End
          End
          If ispllib = '' then
          Do
            Parse var fileline.i firstbit 'ispllib='ispllib
            if pos('*',firstbit) /= 0 Then ispllib = ''
            If ispllib /= '' Then
            Do
              Call Chk_dsns 'ispllib' ispllib
              If rc > dsn_rc then dsn_rc = rc
            End
          End
        End
        If dsn_rc = 0 Then
        Do
          Say 'INFO : All data sets in ISPF.conf exist'
          Say ' '
        End
        If dsn_rc > max_rc then max_rc = dsn_rc
      End
    End
  End
  Else
  Do
    Say 'ERROR: ISPZXENV not found. Check ISPF Gateway is set in PATH statement'
    Say ' '
  End
End

Say ' '
Say 'INFO : Testing ISPF Gateway configuration by running startispf.sh'
Say 'INFO : Invoking startispf.sh with the TSO TIME command'
Say ' '
shellcmd = confpath'startispf.sh TIME'
sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
If sh_rc /= 0 Then
Do
  Call Errors
  Say 'ERROR:' confpath'startispf.sh command failed. See ISPF Gateway log'
  Say ' '
  sh_rc = 8
  If sh_rc > max_rc then max_rc = sh_rc
End
Say 'INFO : Output from ISPF gateway invocation using startispf.sh'
Say ' '
Say Copies('-',80)
Do T = 1 to stdout.0
   Say stdout.T
End
Say Copies('-',80)
Say ''

Return

BFAgent :

Say '------------------------------------------------------------------'
Say '- Checking the configuration of the Rational Build Agent         -'
Say '------------------------------------------------------------------'
Say ' '

shellcmd='find' confpath'startbfa.sh'
sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
If sh_rc /= 0 Then
Do
  Call Errors
  Say ' '
  Say 'ERROR:' confpath'startbfa.sh cannot be accessed'
  Say 'ERROR:  Have you run job BLZCPBTK?'
  sh_rc = 8
  If sh_rc > max_rc then max_rc = sh_rc
End

/* need to get all the environmeent variables. Easiest way  */
/* is to run the shell, without the buildagent startup and  */
/* then trap the output                                     */
filein = confpath'startbfa.sh'
fileline.0 = 0
Address syscall "readfile (filein) fileline."
If rc /= 0 Then
Do
  Call Errors
End
Else
Do
  /* read through file to find bfagent invokation  */
  /* and replace with an env statement to capture  */
  /* environment variables                         */
  Do i = 1 to fileline.0
    /* replace bfagent invocation with env */
    if pos('/bfagent',fileline.i) /= 0 Then
    Do
      BFAinvoke = fileline.i
      fileline.i = 'env'
    End
  End

  /* get invokers home directory for temporary files */
  user = USERID()
  address syscall 'getpwnam (user) pw.'
  homeDir = pw.4
  ivpfile  = homeDir'/tempRTCivp.sh'
  Address syscall "writefile (ivpfile) 755 fileline."
  Say 'RC from writefile : 'rc
  shellcmd = 'tempRTCivp.sh; rm tempRTCivp.sh'
  sh_rc = bpxwunix(shellcmd,,envvars.,stderr.)
  If sh_rc /= 0 Then
  Do
    Call Errors
    Say ' '
    Say 'ERROR: Error running temporary shell script.'
    sh_rc = 8
    If sh_rc > max_rc then max_rc = sh_rc
  End
  Say 'INFO : startbfa environment variables set to:'
  Say '---------------------------------------------'
  Do i = 1 to envvars.0
     Say 'INFO : 'envvars.i
     Parse var envvars.i varname '=' varvalue
     /* validate what the environment variable is set to */
     Select
       When (Strip(varname) = 'JAZZ_USER') Then
       Do
         If varvalue = '@yourUserid@' then
         Do
           Say 'WARN : Jazz userid has not been configured'
           sh_rc = 4
           If sh_rc > max_rc then max_rc = sh_rc
         End
       End
       When (Strip(varname) = 'JAZZ_PASSWORD_FILE') Then
       Do
         If varvalue = '@yourPasswordFile@' Then
         Do
           Say 'WARN : Jazz password file has not been configured'
           sh_rc = 4
           If sh_rc > max_rc then max_rc = sh_rc
         End
         Else
         Do
           shellcmd='find' varvalue
           sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
           If sh_rc /= 0 Then
           Do
             Say 'ERROR: password file 'varvalue' not found or accessible'
             sh_rc = 8
             If sh_rc > max_rc then max_rc = sh_rc
           End
           Else
             Say 'INFO : Password file exists'
         End
       End
       When (Strip(varname) = 'BLD_TOOLKIT') Then
       Do
         shellcmd='find' varvalue
         sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
         If sh_rc /= 0 Then
         Do
           Say 'ERROR: Build Toolkit directory not found'
           sh_rc = 8
           If sh_rc > max_rc then max_rc = sh_rc
         End
         Else
           Say 'INFO : Build Toolkit directory exists'
       End
       When (Strip(varname) = 'STEPLIB') Then
       Do
         If varvalue = '@stepLib@' Then
         Do
           Say 'WARN : STEPLIB not set. Consider commenting out if not required'
           sh_rc = 4
           If sh_rc > max_rc then max_rc = sh_rc
         End
       End
       When (Strip(varname) = '_CMDSERV_BASE_HOME') Then
       Do
         shellcmd='find' varvalue
         sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
         If sh_rc /= 0 Then
         Do
           Say 'ERROR: ISPF Gateway directory not found'
           sh_rc = 8
           If sh_rc > max_rc then max_rc = sh_rc
         End
         Else
           Say 'INFO : ISPF Gateway directory exists'
       End
       When (Strip(varname) = 'JAVA_HOME') Then
       Do
         shellcmd='find' varvalue
         sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
         If sh_rc /= 0 Then
         Do
           Say 'ERROR: Java home directory not found'
           sh_rc = 8
           If sh_rc > max_rc then max_rc = sh_rc
         End
         Else
           Say 'INFO : Java home directory exists'
       End
       When (Strip(varname) = 'ANT_HOME') Then
       Do
         shellcmd='find' varvalue
         sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
         If sh_rc /= 0 Then
         Do
           Say 'ERROR: ANT location in build Toolkit not found'
           sh_rc = 8
           If sh_rc > max_rc then max_rc = sh_rc
         End
         Else
           Say 'INFO : ANT home directory exists'
       End
       When (Strip(varname) = 'SCM_WORK') Then
       Do
         shellcmd='find' varvalue
         sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
         If sh_rc /= 0 Then
         Do
           Say 'ERROR: SCM work directory not found'
           sh_rc = 8
           If sh_rc > max_rc then max_rc = sh_rc
         End
         Else
           Say 'INFO : SCM work directory exists'
       End
       Otherwise
         NOP
     End
     Say ''
  End
  Say '---------------------------------------------'
  Say ''
  /* check the location of the bfagent and config file */
  Say 'INFO : Checking build agent invocation'
  Say ''
  Parse var BFAinvoke bfagent '-s -f ' bfconf
  Say 'INFO : Build Forge Agent command set to 'bfagent
  shellcmd='find' bfagent
  sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
  If sh_rc /= 0 Then
  Do
    Say 'ERROR: Build forge Agent executable not found'
    sh_rc = 8
    If sh_rc > max_rc then max_rc = sh_rc
  End
  Say 'INFO : Build Forge config file set to 'bfconf
  shellcmd='find' bfconf
  sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
  If sh_rc /= 0 Then
  Do
    Say 'ERROR: Build forge Agent config file not found'
    sh_rc = 8
    If sh_rc > max_rc then max_rc = sh_rc
  End
  Say ' '
  Say 'INFO : Checking build agent started task (BLZBFA.*) '
  Say ''
  XX=OUTTRAP('RACF.')
/*Address TSO 'RLIST STARTED BLZBFA.* ALL STDATA'*/
  Address TSO 'RLIST STARTED BFAGENT.* ALL STDATA'
  XX=OUTTRAP('OFF')
  If Pos('NOT FOUND',RACF.1) /= 0 then
  Do
    Say 'INFO : Build Forge started task not created with default name'
    Say 'INFO : Consider using a started task to invoke the build forge'
    Say 'INFO : agent. See job hlq.SBLZSAMP(BLZBFA).'
  End
  Else
  Do
    /* Check and see what userid the STC will use */
    confgrp = ''
    Say 'INFO : Build Forge started task of BLZBFA.* found'
    Do i = 1 to RACF.0 while Substr(RACF.i,1,6) /= 'STDATA'
    End
    Do j = i to RACF.0 While (confgrp = '')
      If Substr(RACF.j,1,5) = 'USER=' then
      Do
        Parse var RACF.j 'USER=' STCUSER
        Say 'INFO : Build Forge started task will use the authority of ' ||,
                   'user : 'Strip(STCUSER)
        /* check the owner of /etc/jazz/ccm and make sure that this */
        /* userid is in that group                                  */
        /* Get the GID */
        Say ''
        Say 'INFO : Checking if the started task userid is in the correct group'
        shellcmd = 'ls -an 'confpath
        sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
        If sh_rc = 0 Then
        Do
          Do T = 1 to stdout.0
            parse var stdout.T perms 12 . 16 uid 25 gid,
                               34 size 42 date 55 filename
            If substr(perms,1,1) = 'd' &,
               substr(filename,1,1) = '.' then
            Do
              /* now list (if we can) all the groups for this GID */
              XX=OUTTRAP('RACF.')
              Address TSO 'SR CLASS(GROUP) GID('strip(gid)')'
              XX=OUTTRAP('OFF')
              If Pos('The GID keyword requires',RACF.1) Then
              Do
                Say 'WARN : Problem searching the RACF database'
                Say 'WARN : 'RACF.1
                Say 'WARN : Will not be able to check if multiple ' ||,
                           'groups are assicated to the same GID'
                Say 'INFO : Attempting to use the group name associated ' ||,
                           'with the config directory'
                shellcmd = 'ls -alE 'confpath
                sh_rc = bpxwunix(shellcmd,,stdout.,stderr.)
                If sh_rc = 0 Then
                Do
                  Do T = 1 to stdout.0
                    parse var stdout.T perms 13 ext 17 . 21 owner 29 group,
                                       38 size 47 date 60 filename
                    If substr(perms,1,1) = 'd' &,
                       substr(filename,1,1) = '.' then
                    Do
                      confgrp = group
                      leave
                    End
                  End
                End
              End
              Else
              Do
                If RACF.0 > 1 Then
                Do
                  Say 'WARN : GID assigned to config directory is not unique'
                  Say '       Check that Started task userid is connected to'
                  Say '       the correct group. Currently assigned to groups:'
                  Do r = 1 to RACF.0
                    Say '        'RACF.r
                  End
                End
                Else
                  confgrp = RACF.1
              End
              leave
            End
          End
        End
      End
    End
    If confgrp /= '' Then
    Do
      confgrp = Strip(confgrp)
      Say 'INFO : Group 'confgrp' associated with directory 'confpath
      XX=OUTTRAP('RACF.')
      Address TSO 'LG 'confgrp
      XX=OUTTRAP('OFF')
      Do i = 1 to RACF.0 while Pos('USER(S)=',RACF.i) = 0
      End
      /* loop through user records and look for STC userid */
      userFnd = 'no'
      Do j = i to RACF.0 while (userFnd = 'no')
        Parse var RACF.j . 7 user 15 .
        If Strip(user) = Strip(STCUSER) Then
          UserFnd = 'yes'
      End
      If userFnd = 'No' Then
        Say 'WARN : User 'Strip(STCUSER)' not found in group 'confgrp
      Else
        Say 'INFO : User 'Strip(STCUSER)' found in group 'confgrp
      Say ''
    End
  End
End

Return

ISPFClient :

Say ' '
Say '------------------------------------------------------------------'
Say '- Checking the configuration of the ISPF Client                  -'
Say '------------------------------------------------------------------'
Say ' '


Return

Chk_dsns:

  Parse arg dd dsnList

  /* check that RTC product message library has been added */
  If dd = 'ispmlib' then
    If Pos('SBLZM',dsnlist) = 0 Then
      Say 'ERROR: No RTC product message library in ISPMLIB concatenation'

  /* check that RTC product load library has been added    */
  If dd = 'ispllib' then
    If Pos('SBLZLOAD',dsnlist) = 0 Then
      Say 'ERROR: No RTC product message library in ISPLLIB concatenation'

  Do While dsnList /= ''
    parse var dsnList dsn ',' dsnList
    dsn = "'"Strip(dsn)"'"
    ListdsiRC = Listdsi(dsn)
    If ListdsiRC /= 0 Then
    Do
      If (SYSREASON = 24 | SYSREASON = 5) Then
        Say 'ERROR: Data set 'dsn' allocated to 'dd' does not exist'
      Else
        Say 'ERROR: Problem with LISTDSI function. SYSREASON = 'SYSREASON
      If ListdsiRC > max_listdsi_rc then max_listdsi_rc =  ListdsiRC
    End
  End

Return max_listdsi_rc

Errors:

  If stderr.0 > 0 Then
  Do
    Say '----- STDERR ----'
    Do t = 1 to stderr.0
      Say stderr.t
    End
    Say '--- End STDERR---'
    Say ' '
  End

Return
